@@ -52,7 +52,7 @@ void *util_ldap_url_node_copy(util_ald_cache_t *cache, void *c)
 
     if (node) {
         if (!(node->url = util_ald_strdup(cache, n->url))) {
-            util_ald_free(cache, node);
+            util_ald_free(cache, node->url);
             return NULL;
         }
         node->search_cache = n->search_cache;
@@ -79,7 +79,7 @@ void util_ldap_url_node_free(util_ald_cache_t *cache, void *n)
 void util_ldap_url_node_display(request_rec *r, util_ald_cache_t *cache, void *n)
 {
     util_url_node_t *node = n;
-    char date_str[APR_CTIME_LEN+1];
+    char date_str[APR_CTIME_LEN];
     const char *type_str;
     util_ald_cache_t *cache_node;
     int x;
@@ -218,7 +218,7 @@ void util_ldap_search_node_free(util_ald_cache_t *cache, void *n)
 void util_ldap_search_node_display(request_rec *r, util_ald_cache_t *cache, void *n)
 {
     util_search_node_t *node = n;
-    char date_str[APR_CTIME_LEN+1];
+    char date_str[APR_CTIME_LEN];
 
     apr_ctime(date_str, node->lastbind);
 
@@ -259,12 +259,14 @@ void *util_ldap_compare_node_copy(util_ald_cache_t *cache, void *c)
     if (node) {
         if (!(node->dn = util_ald_strdup(cache, n->dn)) ||
             !(node->attrib = util_ald_strdup(cache, n->attrib)) ||
-            !(node->value = util_ald_strdup(cache, n->value))) {
+            !(node->value = util_ald_strdup(cache, n->value)) ||
+            ((n->subgroupList) && !(node->subgroupList = util_ald_sgl_dup(cache, n->subgroupList)))) {
             util_ldap_compare_node_free(cache, node);
             return NULL;
         }
         node->lastcompare = n->lastcompare;
         node->result = n->result;
+        node->sgl_processed = n->sgl_processed;
         return node;
     }
     else {
@@ -275,6 +277,8 @@ void *util_ldap_compare_node_copy(util_ald_cache_t *cache, void *c)
 void util_ldap_compare_node_free(util_ald_cache_t *cache, void *n)
 {
     util_compare_node_t *node = n;
+
+    util_ald_sgl_free(cache, &(node->subgroupList));
     util_ald_free(cache, node->dn);
     util_ald_free(cache, node->attrib);
     util_ald_free(cache, node->value);
@@ -284,8 +288,10 @@ void util_ldap_compare_node_free(util_ald_cache_t *cache, void *n)
 void util_ldap_compare_node_display(request_rec *r, util_ald_cache_t *cache, void *n)
 {
     util_compare_node_t *node = n;
-    char date_str[APR_CTIME_LEN+1];
+    char date_str[APR_CTIME_LEN];
     char *cmp_result;
+    char *sub_groups_val;
+    char *sub_groups_checked;
 
     apr_ctime(date_str, node->lastcompare);
 
@@ -299,6 +305,20 @@ void util_ldap_compare_node_display(request_rec *r, util_ald_cache_t *cache, voi
         cmp_result = apr_itoa(r->pool, node->result);
     }
 
+    if(node->subgroupList) {
+        sub_groups_val = "Yes";
+    }
+    else {
+        sub_groups_val = "No";
+    }
+
+    if(node->sgl_processed) {
+        sub_groups_checked = "Yes";
+    }
+    else {
+        sub_groups_checked = "No";
+    }
+
     ap_rprintf(r,
                "<tr valign='top'>"
                "<td nowrap>%s</td>"
@@ -306,12 +326,16 @@ void util_ldap_compare_node_display(request_rec *r, util_ald_cache_t *cache, voi
                "<td nowrap>%s</td>"
                "<td nowrap>%s</td>"
                "<td nowrap>%s</td>"
+               "<td nowrap>%s</td>"
+               "<td nowrap>%s</td>"
                "</tr>",
                node->dn,
                node->attrib,
                node->value,
                date_str,
-               cmp_result);
+               cmp_result,
+               sub_groups_val,
+               sub_groups_checked);
 }
 
 /* ------------------------------------------------------------------ */
@@ -396,27 +420,29 @@ apr_status_t util_ldap_cache_init(apr_pool_t *pool, util_ldap_state_t *st)
     apr_status_t result;
     apr_size_t size;
 
-    if (st->cache_file) {
-        /* Remove any existing shm segment with this name. */
-        apr_shm_remove(st->cache_file, st->pool);
-    }
+    if (st->cache_bytes > 0) {
+        if (st->cache_file) {
+            /* Remove any existing shm segment with this name. */
+            apr_shm_remove(st->cache_file, st->pool);
+        }
 
-    size = APR_ALIGN_DEFAULT(st->cache_bytes);
+        size = APR_ALIGN_DEFAULT(st->cache_bytes);
 
-    result = apr_shm_create(&st->cache_shm, size, st->cache_file, st->pool);
-    if (result != APR_SUCCESS) {
-        return result;
-    }
+        result = apr_shm_create(&st->cache_shm, size, st->cache_file, st->pool);
+        if (result != APR_SUCCESS) {
+            return result;
+        }
 
-    /* Determine the usable size of the shm segment. */
-    size = apr_shm_size_get(st->cache_shm);
+        /* Determine the usable size of the shm segment. */
+        size = apr_shm_size_get(st->cache_shm);
 
-    /* This will create a rmm "handler" to get into the shared memory area */
-    result = apr_rmm_init(&st->cache_rmm, NULL,
-                          apr_shm_baseaddr_get(st->cache_shm), size,
-                          st->pool);
-    if (result != APR_SUCCESS) {
-        return result;
+        /* This will create a rmm "handler" to get into the shared memory area */
+        result = apr_rmm_init(&st->cache_rmm, NULL,
+                              apr_shm_baseaddr_get(st->cache_shm), size,
+                              st->pool);
+        if (result != APR_SUCCESS) {
+            return result;
+        }
     }
 
 #endif
